@@ -16,9 +16,9 @@
     import ogParser
     import ogAST
 
-    process = ogParser.parseProcessDefinition(filename='test.pr')
+    ast = ogParser.parse_pr(files=['file1.pr', ...], string='...')
 
-    -> process is of type Process (see below)
+    -> ast is of type AST (see below)
 
     Design note:
         classes are used here only for the purpose of convenient
@@ -127,7 +127,6 @@ class Decision(object):
         self.pos_y = 0
         self.width = 100
         self.height = 70
-        self.abs_x, self.abs_y = None, None
         # kind can be 'any', 'informal_text', or 'question'
         self.kind = None
         # question is an Expression
@@ -160,7 +159,6 @@ class Answer(object):
         self.pos_y = 0
         self.width = 100
         self.height = 35
-        self.abs_x, self.abs_y = None, None
         # one of 'closed_range' 'constant' 'open_range' 'else' 'informal_text'
         self.kind = None
         # informalText is a string, when kind == 'informal_text'
@@ -196,7 +194,6 @@ class Task(object):
         self.pos_y = 0
         self.width = 100
         self.height = 50
-        self.abs_x, self.abs_y = None, None
         # one of 'informal_text', 'assign'
         self.kind = None
         # list of 'assign'-kind Expression
@@ -223,7 +220,6 @@ class Output(object):
         self.pos_y = 0
         self.width = 100
         self.height = 50
-        self.abs_x, self.abs_y = None, None
         self.line = None
         self.charPositionInLine = None
         # one of 'procedure_call', 'output'
@@ -250,7 +246,6 @@ class Terminator(object):
         self.pos_y = None
         self.width = 100
         self.height = 50
-        self.abs_x, self.abs_y = None, None
         self.line = None
         self.charPositionInLine = None
         # one of 'next_state' 'join' 'stop'
@@ -280,7 +275,6 @@ class Label(object):
         self.pos_y = 0
         self.width = 100
         self.height = 50
-        self.abs_x, self.abs_y = None, None
         self.line = None
         self.charPositionInLine = None
         # optional hyperlink
@@ -304,16 +298,37 @@ class Transition(object):
         self.terminator = None
 
 
+class Floating_label(object):
+    ''' AST Entry for a floating label '''
+    def __init__(self):
+        ''' Initialize the floatinglabel attributes '''
+        self.inputString = 'Here'
+        # optional hyperlink
+        self.hyperlink = None
+        self.pos_x, self.pos_y = 0, 0
+        self.width, self.height = 100, 50
+        self.line = None
+        self.charPositionInLine = None
+        self.transition = None
+        # list of terminators
+        self.terminators = []    
+    
+    def __repr__(self):
+        ''' Debug output for a label (used by code generator) '''
+        return 'CONNECTION {label} ({l},{c})'.format(label=self.inputString,
+                l=self.line, c=self.charPositionInLine)
+
+
 class Input(object):
     ''' AST Entry for the INPUT symbol '''
     def __init__(self):
         ''' Initialize the Input attributes '''
+        # inputString is the user text, it can contain several inputs
         self.inputString = 'msgName'
         self.pos_x = 0
         self.pos_y = 0
         self.width = 100
         self.height = 50
-        self.abs_x, self.abs_y = None, None
         self.line = None
         self.charPositionInLine = None
         # provided clause (not supported yet)
@@ -323,14 +338,16 @@ class Input(object):
         # List of input parameters (strings: each param is a variable)
         # (If there are several inputs in the symbol, there are no parameters)
         self.parameters = []
-        # list of inputs and corresponding transitions
-        # dictionnary: {'inputName': transitionRef, ... }
-        # transitionRef is an index of the process.transitions list
-        self.inputlist = {}
+        # list of input signal names
+        self.inputlist = []
+        # transition_id is an index of the process.transitions list
+        self.transition_id = -1
         # optional comment symbol
         self.comment = None
         # optional hyperlink
         self.hyperlink = None
+        # list of terminators following the input symbol
+        self.terminators = []
 
     def __repr__(self):
         ''' Debug output for an INPUT symbol '''
@@ -346,13 +363,14 @@ class Start(object):
         self.pos_y = 0
         self.width = 100
         self.height = 50
-        self.abs_x, self.abs_y = None, None
         # start transition is of type Transition
         self.transition = None
         # optional comment symbol
         self.comment = None
         # optional hyperlink
         self.hyperlink = None
+        # list of terminators following the start symbol
+        self.terminators = []
 
     def __repr__(self):
         ''' Debug output for a START symbol '''
@@ -369,7 +387,6 @@ class Comment(object):
         self.pos_y = 0
         self.width = 100
         self.height = 50
-        self.abs_x, self.abs_y = None, None
         self.line = None
         self.charPositionInLine = None
         # optional hyperlink
@@ -387,13 +404,14 @@ class State(object):
         ''' Used only for rendering backends - not for code generation '''
         # inputString contains possibly several states (and asterisk)
         self.inputString = defName
+        # List of state names contained in the symbol
+        self.statelist = []
         self.line = None
         self.charPositionInLine = None
         self.pos_x = 0
         self.pos_y = 0
         self.width = 100
         self.height = 50
-        self.abs_x, self.abs_y = None, None
         # list of type Input
         self.inputs = []
         # optional comment symbol
@@ -413,6 +431,8 @@ class TextArea(object):
     def __init__(self):
         ''' Text area (raw content for rendering only) '''
         self.inputString = '-- Declare your variables\n\nDCL foo MyInteger;'
+        # DCL variables in the text area {name: sort, ...}
+        self.variables = {}
         self.line = None
         self.charPositionInLine = None
         # Set default coordinates and width/height
@@ -420,7 +440,6 @@ class TextArea(object):
         self.pos_y = 0
         self.width = 170
         self.height = 140
-        self.abs_x, self.abs_y = None, None
         # optional hyperlink
         self.hyperlink = None
 
@@ -430,36 +449,94 @@ class TextArea(object):
                 l=self.line, c=self.charPositionInLine)
 
 
+class Procedure(object):
+    ''' Internal procedure definition '''
+    def __init__(self):
+        ''' Procedure AST default value '''
+        self.inputString = 'Proc'
+        self.line = None
+        self.charPositionInLine = None
+        # Set default coordinates and width/height
+        self.pos_x = self.pos_y = 0
+        self.width = 100
+        self.height = 50
+        # Optional hyperlink
+        self.hyperlink = None
+        # Local variables dictionnary (see Process)
+        self.variables = {}
+        # Inherited variables from all levels above
+        self.global_variables = {}
+        # Formal parameters - list of dict:
+        # [{'name': str, 'direction': 'in'/'out', 'type': str}]
+        self.fpar = []
+        # start, states, terminators, text areas, floating_labels (see Process)
+        self.start = None
+        self.states = []
+        self.terminators = []
+        self.textAreas = []
+        self.floating_labels = []
+        # Inherited procedures and operators
+        self.procedures = {}
+        self.operators = {}
+        # inner procedures and operators (see Process for format)
+        self.inner_procedures = []
+        self.inner_operators = []
+        # mapping, transitions: see Process
+        self.mapping = {}
+        self.transitions = []
+        # Determine if a procedure is externally defined
+        self.external = False
+        # Optional comment
+        self.comment = None
+
+
 class Process(object):
     ''' SDL Process entry point '''
     def __init__(self):
-        ''' Initializes the AST for a complete SDL process '''
+        ''' Process AST default values '''
         self.processName = None
+        # Optional filename containing this process (PR file)
+        self.filename = None
+        # Process parent context (BLOCK)
+        self.parent = None
+        # A process can be referenced (externally defined)
+        self.referenced = False
         # variables: dictionnary: {variable1Name:asn1SccType, ...}
         # WARNING default value is not supported yet
         self.variables = {}
+        # global variables can be used to inherit variables
+        self.global_variables = {}
 
         # dataView: complete AST of the ASN.1 types
         self.asn1Modules = None
         self.dataView = None
 
         # input and output signal lists: {'signal': definition from iv.py, ...}
-        self.inputSignals = {}
-        self.outputSignals = {}
+        # input and output signal lists:
+        # [{'name': str, 'type': str, 'direction':'in'/'out'}]
+        self.input_signals = []
+        self.output_signals = []
 
         # start, states, and terminators are useful for rendering backends
         self.start = None
         self.states = []
         self.terminators = []
 
+        # List of Floating_label
+        self.floating_labels = []
+
         # list of operators (not supported) and procedures
         self.operators = {}
-        # procedures: same form as outputlist:
+        # procedures: same form as outputlist: NOT ANYMORE
         #     {'procedureName': definition from iv.py, ...}
         # only supports procedures as sync RI specified in the
         # TASTE interface view
-        # use operators for other purposes
-        self.procedures = {}
+        # use operators or inner_procedures for other purposes
+        #self.procedures = {}
+        # list of class Procedure (external procedures)
+        self.procedures = []
+        # User-defined procedures: list of ogAST.Procedure
+        self.inner_procedures = []
 
         # list of type TextArea. Useful for rendering only
         # use 'variables', 'operators' and 'procedures' to get useful content
@@ -467,8 +544,76 @@ class Process(object):
 
         # The Mapping structure should be used for code generation backends
         # dictionnary: {'stateName': [class Input1, Input2,...], ...}
-        # then class Input contains the inputs list and corresponding transition
+        # then Input contains the inputs list and corresponding transition
         self.mapping = {}
 
         # list of type Transition - use 'mapping' to map index to inputs/states
         self.transitions = []
+
+
+class Block(object):
+    ''' AST for a BLOCK entity '''
+    def __init__(self):
+        '''
+            Blocks can contain signal declarations, signalroutes,
+            connections, inner block definitions, and process
+            definitions.
+        '''
+        # Block name
+        self.name = ''
+        # Parent block or system
+        self.parent = None
+        # list of signal declarations - see class System for the structure
+        self.signals = []
+        # signalroutes: see self.channels in class System for the structure
+        self.signalroutes = []
+        # connections are between a channel and signalroute
+        # [{'channel': str, 'signalroute': str}]
+        self.connections = []
+        # list of class Block
+        self.blocks = []
+        # list of class Process
+        self.processes = []
+
+
+class System(object):
+    ''' AST for a SYSTEM entity '''
+    def __init__(self):
+        '''
+            Create system default values
+            contains blocks, procedures, channels and signal declarations
+        '''
+        # System name
+        self.name = ''
+        # list of SIGNAL declarations: [{'name': str, 'type': str}]
+        # (Supporting only one parameter)
+        self.signals = []
+        # list of ogAST.Procedure
+        self.procedures = []
+        # channels: [{'name':str, 'routes':[{'source': str, 'dest': str, 
+        # 'signals': ['sig1', .. ]}]
+        self.channels = []
+        self.blocks = []
+
+
+class AST(object):
+    ''' Top-level AST entry point '''
+    def __init__(self):
+        '''
+            AST entries are systems, processes, and USE clauses
+            (cf. ANTLR grammar, production "pr_file")
+        '''
+        # Set of input files the AST was created from
+        self.pr_files = set()
+        # USE clauses: list of strings (eg. "DataView")
+        self.use_clauses = []
+        # Optional list of ASN.1 filenames (defined in CIF of USE clauses)
+        self.asn1_filenames = []
+        # Refs to the ASN.1 dataview AST (set with USE clauses)
+        self.dataview = None
+        self.asn1Modules = None
+
+        # List of System
+        self.systems = []
+        # List of Process
+        self.processes = []
